//+------------------------------------------------------------------+
//|                                                       QX_BMA.mq4 |
//|                                   Copyright  2008, Quantum_Xcel |
//|                                       http://www.quantumxcel.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Quantum_Xcel"
#property link      "http://www.quantumxcel.com"
//
// Version History:
//   0.01 2008.05.19.
//Words best on GBP/USD Daily Chart
//
#include <stderror.mqh>
#include <stdlib.mqh>
//
#define MAGIC_NUMBER 51508002
//
extern string EACopyright = "QX_BMAv0.01 Copyright  2008 QuantumXcel";
extern int    MAPeriod    = 1;
extern int    TakeProfit  = 55;
extern int    StopLoss    = 25;
extern int    Slippage    = 5;
// The taken risk in percentage.
extern double Risk = 20;
// Size of the lot for one trade. The EA will ignore this parameter if Risk > 0.
extern double LotSize = 0;
extern bool ShowAlert = False;
extern string EATradingTime = "====== Trading hours in HH:MM GMT ======";
extern string StartTime = "00:10";
extern string EndTime   = "23:55";
//
int    OP_NONE;
string Pairs;
int    CurTF;
int    CurDigits;
double CurPoint;
int    LastError;
int    StartHour;
int    StartMinute;
int    EndHour;
int    EndMinute;
//
int MinBars = 100;
//
int init( )
{
  Comment( EACopyright );
  
  datetime StartTrading = StrToTime( StartTime );
  datetime EndTrading   = StrToTime( EndTime );
  
  StartHour    = TimeHour( StartTrading );
  StartMinute  = TimeMinute( StartTrading );
  EndHour      = TimeHour( EndTrading );
  EndMinute    = TimeMinute( EndTrading );
  
  if( !AreParamsOk( True ) )
    return( 0 );
  
  LastError = 0;
  OP_NONE = OP_SELLSTOP + 1;
  Pairs = Symbol( );
  CurTF = Period( );
  SetDigits( );

  return( 0 );
}
//
int deinit( )
{
  ObjectsDeleteAll( );
  
  return( 0 );
}
//
int start( )
{
  if( !AreParamsOk( False ) )
  
    return( 0 );

  ManageOrders( );
  OpenOrders( );
  CloseOrders( );
    
  return( 0 );
}
//
int OpenOrders( )
{
  int OpenSignals = GetOpenSignals( );
  
  if( OpenSignals == OP_BUY ) 
    OpenWait( OP_BUY, Ask, Ask - StopLoss * CurPoint, Ask + TakeProfit * 
      CurPoint );
  else
  if( OpenSignals == OP_SELL )
    OpenWait( OP_SELL, Bid, Bid + StopLoss * CurPoint, Bid - TakeProfit * 
      CurPoint );
  
  return( 0 );
}
//
bool IsTradingTime( )
{
  bool RetValue  = False;
  int  CurHour   = Hour( );
  int  CurMinute = Minute( );
  
  if( StartHour > EndHour )
    RetValue = CurHour >= StartHour && CurHour <= EndHour;
  else  
  if( StartHour < EndHour )
    RetValue = CurHour >= StartHour || CurHour <= EndHour;
  
  if( CurHour == StartHour )
    RetValue = CurMinute >= StartMinute;
  if( CurHour == EndHour )  
    RetValue = CurMinute <= EndMinute;

  return( RetValue );
}
//
bool IsEnoughBars( )
{
  return( iBars( Pairs, CurTF ) >= MinBars );
}
//
bool AreParamsOk( bool ShowMessage )
{
  if( MAPeriod < 0 || MAPeriod > 9999 )
  {
    if( ShowMessage )
      Alert( "The valid value of MAPeriod is 0-9999." );
      
    return( False );
  }
  if( TakeProfit < 0 || TakeProfit > 9999 )
  {
    if( ShowMessage )
      Alert( "The valid value of TakeProfit is 0-9999." );
      
    return( False );
  }
  if( StopLoss < 0 || StopLoss > 9999 )
  {
    if( ShowMessage )
      Alert( "The valid value of StopLoss is 0-9999." );
      
    return( False );
  }
  if( Risk < 0 || Risk > 99.0 )
  {
    if( ShowMessage )
      Alert( "The valid value of Risk is 0-99." );
      
    return( False );
  }
  if( Risk == 0 && LotSize <= 0 )
  {
    if( ShowMessage )
      Alert( "The valid value of LotSize is > 0." );
      
    return( False );
  }
  
  if( ShowMessage )
  {
    string strStartHour   = StartHour;
    string strStartMinute = StartMinute;
    string strEndHour     = EndHour;
    string strEndMinute   = EndMinute;
    
    if( StartHour < 10 )
       strStartHour = "0" + strStartHour;
    if( StartMinute < 10 )
      strStartMinute = "0" + strStartMinute;
    if( EndHour < 10 )
      strEndHour = "0" + strEndHour;
    if( EndMinute < 10 )
      strEndMinute = "0" + strEndMinute;

    Alert( "Trading hours (GMT) are between ", strStartHour, ":", strStartMinute, 
      " and ", strEndHour, ":", strEndMinute, "." );
  }  

  return( True );
}
//
int GetOrderCounts( )
{
  int Orders = 0;
 
  for( int i = OrdersTotal( ) - 1; Orders == 0 && i >= 0; i-- )
  {
    OrderSelect( i, SELECT_BY_POS );
    if( OrderSymbol( ) == Pairs && OrderMagicNumber( ) == MAGIC_NUMBER )
      Orders++;
  }
 
  return( Orders );
}
//
bool IsOpenAllowed( )
{
  return( IsTradeAllowed( ) && IsEnoughBars( ) && GetOrderCounts( ) == 0 && 
    IsTradingTime( ) );
}
//
int GetOpenSignals( )
{
  if( IsOpenAllowed( ) )

    return( GetRHOpenSignals( ) );
      
  return( OP_NONE );   
}
//
int GetRHOpenSignals( )
{
  // Temporary strategy lol;
  
  int    Op         = OP_NONE;
  double MA         = (Open[1]+Close[1])/2;
  double ClosePrice = Open[ 1 ];
  
  // Better to limit the digits if you want to compare 2 floating points.
  MA = NormalizeDouble( MA, CurDigits );
  
  if( MA > ClosePrice )
    Op = OP_BUY;
  else
  if( MA < ClosePrice )
    Op = OP_SELL;
  
  return( Op );
}
//
bool IsCloseSignals( )
{
  double Pips    = CalcProfit( );
  bool   IsSLHit = StopLoss > 0 && Pips <= -StopLoss;
  bool   IsTPHit = TakeProfit > 0 && Pips >= TakeProfit;
   
  return( IsSLHit || IsTPHit );
}
//
int ManageOrders( )
{
  for( int i = 0; i < OrdersTotal( ); i++ )
  {
    OrderSelect( i, SELECT_BY_POS );
    if( OrderSymbol( ) == Pairs && OrderMagicNumber( ) == MAGIC_NUMBER )
    { }  
  }
    
  return( 0 );
}
//
int CloseOrders( )
{
  for( int i = OrdersTotal( ) - 1; i >= 0; i-- )
  {
    OrderSelect( i, SELECT_BY_POS );
    if( OrderSymbol( ) == Pairs && OrderMagicNumber( ) == MAGIC_NUMBER )
      CloseOrder( );
  }
 
  return( 0 );
}
//
double CalcProfit( )
{
  double Pips = 0.0;
  int    Op   = OrderType( );

  if( Op == OP_BUY )
    Pips = MathFloor( ( Bid - OrderOpenPrice( ) ) / CurPoint ); 
  else
  if( Op == OP_SELL )
    Pips = MathFloor( ( OrderOpenPrice( ) - Ask ) / CurPoint ); 
  
  return( Pips );
}
//
bool CloseOrder( )
{
  double Price;
  int    Op;
   
  if( IsCloseSignals( ) )
  {
    Op = OrderType( );
    if( Op == OP_BUY )
      Price = Bid;
    else
    if( Op == OP_SELL )
      Price = Ask;
 
    return( CloseWait( Price ) );
  } 
  
  return( True );
}
//
double CalcLots( )
{
  if( Risk == 0 )
    
    return( LotSize );
  
  double Lots;
  
  double MinLots = MarketInfo( Pairs, MODE_MINLOT );
  double MaxLots = MarketInfo( Pairs, MODE_MAXLOT );
  double LotStep = MarketInfo( Pairs, MODE_LOTSTEP );
  double LotBase = MarketInfo( Pairs, MODE_LOTSIZE );
    
  Lots = NormalizeDouble( AccountFreeMargin( ) * Risk * 0.01 * 
    AccountLeverage( ) / LotBase, 2 );
  Lots = NormalizeDouble( Lots / LotStep, 0 ) * LotStep;
  Lots = MathMax( Lots, MinLots );
  Lots = MathMin( Lots, MaxLots );
  
  return( Lots );
}
//
int OpenWait( int Op, double Price, double SL, double TP )
{
  int Ticket = -1;
  
  Price = NormalizeDouble( Price, CurDigits );
  SL = NormalizeDouble( SL, CurDigits );
  TP = NormalizeDouble( TP, CurDigits );
  while( IsTradeContextBusy( ) ){}
  while( Ticket == -1 )
    Ticket = OrderSend( Pairs, Op, CalcLots( ), Price, Slippage, SL, TP, 
      "RH EA.", MAGIC_NUMBER );

  if( ShowAlert )
  {
    if( Op == OP_BUY )
      Alert( "Open buy at ", Price, "." );  
    else
    if( Op == OP_SELL )
      Alert( "Open sell at ", Price, "." );  
  } 
  
  return( Ticket );  
}   
//
bool CloseWait( double Price )
{
  Price = NormalizeDouble( Price, CurDigits );
  while( IsTradeContextBusy( ) ){}
  if( ShowAlert )
    Alert( "Close order at ", Price, " slippage ", Slippage, "." );  

  bool RetValue = OrderClose( OrderTicket( ), OrderLots( ), Price, Slippage );   
  
  return( RetValue );
}   
//
int SetDigits( )
{
  CurDigits = Digits;
  CurPoint  = Point;
  if( CurDigits % 2 == 1 )
  {
    CurDigits--;
    CurPoint = CurPoint * 10;
  }  
}   
//